import numpy as np

def F(x,y):
    return np.array([x**2+y**2-4,x*y-1])

# I dette tilfellet er det enkleste å bruke formelen for inversen til en 2*2 matrise.
# Som regel er ikke inversen så lett tilgjengelig som her, da ville vi i stedet ha løst et likningssett for hvert steg i metoden

def J_inv(x,y):
    return 1/(2*x**2-2*y**2)*np.array(([x,-2*y],[-y,2*x]))

# Når vi har mer enn én dimensjon må vi ta i bruk et nytt mål for å beregne hvor stor forbedringen er for hvert steg.
# En måte å gjøre det på er å bruke den evklidiske distansen mellom de nye koordinatene og de fra forrige steg,
# hvilket er det som er implementert i koden her.

def newton(x,y,tol):
    d = 1 # differansen mellom estimatet for n+1 og n.
    n = 0 # antall iterasjoner
    xp, yp = x,y # verdier fra forrige steg så man kan regne ut et feilestimat
    while tol<d:
        vec = np.array([x,y])-J_inv(x,y)@F(x,y) # vektor med oppdaterte x- og y-verdier
        x = vec[0]
        y = vec[1]
        d = np.linalg.norm(np.array([x,y])-np.array([xp,yp])) # regner ut lengden mellom de to vektorene.
        n = n+1
        xp,yp = x,y
    return x,y,n

if __name__ == "__main__":
    tol = 10.0 ** -14
    x0 = 2
    y0 = 0
    x,y,n = newton(x0,y0,tol)
    print("Løsning:",np.array([x,y]))
    print("F(x,y) =",F(x,y))
    x_e = np.sqrt(2+np.sqrt(3)) # eksakte løsninger
    y_e = np.sqrt(4-x_e**2)
    print("Feil i svaret:",np.linalg.norm(np.array([x,y])-np.array([x_e,y_e])))
    print("Antall iterasjoner:",n)

    # Oppfølgningsspørsmål: Hva blir løsningen dersom man starter fra x0 = 0, y0 = 2?